//======================================================= file = common.h =====
//=  Include file for VOQ switch simulation model                             =
//=============================================================================
//=  Notes:  1) This file contains                                            =
//=               - Constants                                                 =
//=               - Common (external) variables                               =
//=               - Common (external) CSIM types                              =
//=               - Function prototypes                                       =
//=          2)  Basic time unit is the millisecond                           =
//=---------------------------------------------------------------------------=
//=  History:  KJC/KY (02/20/01) - Genesis (from voq_1c 1.02)                 =
//=============================================================================
#pragma warn -pro                     // Disable function prototype warning
#pragma warn -stu                     // Disable undefined structure warning
#pragma warn -rch                     // Disable unreachable code warning
#pragma warn -ccc                     // Disable condition always true warning

//----- Conditional definitions -----------------------------------------------
#if defined(MAIN)                     // Determine if main or external
  #define EXTERN
#endif
#if defined(EXTERNAL)
  #define EXTERN extern
#endif

//----- Constants -------------------------------------------------------------
#define VERSION "1.00 <02/20/01>"     // Version number
#define OUT_FILE     "result.txt"     // Name of output file

#define FALSE                  0      // FALSE = 0
#define TRUE                   1      // TRUE = 1
#define PIM                    1      // PIM arbitration
#define ISLIP                  2      // ISLIP arbitration
#define PPVOQ                  3      // PP-VOQ arbitration

#define MEDIA_RATE        10.0e6      // Media rate in bits per millisec
#define CELL_SIZE            100      // Switch core cell size in bytes
#define SPEED_UP               1      // Switch core speed-up factor
#define N                     16      // Number of input ports in the switch
#define NUM_ITER               4      // Number of PIM or iSLIP iterations
#define ARBIT_TYPE         ISLIP      // Defines arbitration type (PIM, ISLIP,
                                      // and PPVOQ are valid)
#define GEN_FLAG               1      // 1 for gen_poisson(), 2 for gen_ipp(),
                                      // 3 for gen_cbr(), 4 for gen_square()

//----- Overall external variables --------------------------------------------
EXTERN double    Sim_time;            // Simuation time
EXTERN int       Inport_count[N];     // Assertable inport counter semaphore
EXTERN int       Outport_count[N];    // Assertable outport counter semaphore

//----- External variables for gen_poisson() ----------------------------------
EXTERN double    Lambda1;             // Arrival rate of packets in pkts/ms

//----- External variables for gen_ipp() --------------------------------------
EXTERN double    Lambda2;             // Arrival rate of packets in pkts/ms
EXTERN double    Alpha;               // Transition rate from "off" to "on"
EXTERN double    Beta;                // Transition rate from "on" to "off"

//----- External variables for gen_cbr() --------------------------------------
// None (hardwired in gen_cbr())

//----- External variables for gen_square() -----------------------------------
// None (hardwired in gen_square())

//----- CSIM types ------------------------------------------------------------
EXTERN FACILITY Outport_buf[N];       // Outport buffer for reassembled packets
EXTERN EVENT    Q_event[N][N];        // Inport VOQs for packets
EXTERN EVENT    Inport_busy[N];       // Event for inport busy
EXTERN EVENT    Outport_busy[N];      // Event for outport busy
EXTERN EVENT    Xmit_done[N];         // Event for transmit done at outport
EXTERN TABLE    Delay_table[N];       // Delay table for packet delay
EXTERN TABLE    Bytes_table[N];       // Bytes transfered table

//----- Function prototypes ---------------------------------------------------
// Traffic generators
EXTERN void gen_poisson(int in_id);
EXTERN void gen_ipp(int in_id);
EXTERN void gen_cbr(int in_id);
EXTERN void gen_square(int in_id);
EXTERN void (* gen[N])(int in_id);

// Segmenters and input ports
EXTERN void segmenter(int in_id, int out_id, int pkt_len, double org_time);
EXTERN void (* seg[N])(int in_id, int out_id, int pkt_len, double org_time);
EXTERN void inport(int in_id, int out_id, int pkt_len, int tail_flag,
                   double org_time);
EXTERN void (* in[N])(int in_id, int out_id, int pkt_len, int tail_flag,
                      double org_time);

// Output ports
EXTERN void outport(int from_id, int out_id, int pkt_len, int tail_flag,
                    double org_time);
EXTERN void (* out[N])(int from_id, int out_id, int pkt_len, int tail_flag,
                       double org_time);

// Switch arbitrators for PIM, iSLIP, and PP-VOQ
EXTERN void arbit_pim(void);
EXTERN void arbit_islip(void);
EXTERN void arbit_ppvoq(void);

// Run-time simulation status reporter
EXTERN void run_sim(double run_time, int run_num);
