//======================================================= file = inport.c =====
//=  VOQ input port for VOQ switch simulation model                           =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: segmenter() - cell segmenter for input port                    =
//=            inport() - Input port                                          =
//=---------------------------------------------------------------------------=
//=  History:  KJC/KY (02/20/01) - Genesis (from voq_1c 1.02)                 =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//============================================================================
//==  Segmenter for input port                                              ==
//============================================================================
void segmenter(int in_id, int out_id, int pkt_len, double org_time)
{
  int    tail_flag;     // Flag for last packet in a cell
  int    num_cells;     // Number of cells in a packet
  int    i;             // Loop counter

  create("segmenter");

  // Segment a packet into cells and put "cell-ized" packet in source buffer
  tail_flag = FALSE;
  num_cells = pkt_len / CELL_SIZE;
  if ((pkt_len % CELL_SIZE) > 0) num_cells = num_cells + 1;
  for(i=0; i<num_cells; i++)
  {
    if (i == (num_cells - 1)) tail_flag = TRUE;
    in[in_id](in_id, out_id, pkt_len, tail_flag, org_time);
  }
}

//============================================================================
//==  Input port                                                            ==
//============================================================================
void inport(int in_id, int out_id, int pkt_len, int tail_flag, double org_time)
{
  create("inport");

  // Queue the cell in the appropriate VOQ (out_id) in this port (in_id)
  queue(Q_event[in_id][out_id]);

  // Assert that only one transmission occurs at an inport and outport
  assert(Inport_count[in_id] == 0);
  assert(Outport_count[out_id] == 0);

  // Increment counter semphores for an inport and outport pair
  Inport_count[in_id]++;
  Outport_count[out_id]++;

  // Forward the cell to the outport
  hold((8.0 * CELL_SIZE) / (SPEED_UP * MEDIA_RATE));
  out[out_id](in_id, out_id, pkt_len, tail_flag, org_time);
}
