//====================================================== file = outport.c =====
//=  Output port for VOQ switch simulation model                              =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: outport()                                                      =
//=---------------------------------------------------------------------------=
//=  History:  KJC/KY (02/20/01) - Genesis (from voq_1c 1.02)                 =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff
#include <assert.h>     // Needed for assert() macro

//============================================================================
//==  Output port                                                           ==
//============================================================================
void outport(int from_id, int out_id, int pkt_len, int tail_flag,
             double org_time)
{
  create("outport");

  // Decrement counter semphores for an inport and outport pair
  Inport_count[from_id]--;
  Outport_count[out_id]--;

  // Clear port busy events and set transmit done event
  clear(Inport_busy[from_id]);
  clear(Outport_busy[out_id]);
  set(Xmit_done[out_id]);

  // When the last cell for a segmented packet arrives, the cells are considered
  // reassembled and the packet is transmitted on the output link.
  if (tail_flag == TRUE)
  {
    reserve(Outport_buf[out_id]);
    hold((8.0 * pkt_len) / MEDIA_RATE);
    release(Outport_buf[out_id]);

    // Record statistics for packet leaving this port
    record((clock - org_time), Delay_table[out_id]);
    record(pkt_len, Bytes_table[out_id]);
  }

  return;
}
